@@ -9,28 +9,30 @@ import android.content.Intent;
 import android.content.IntentFilter;
 import android.nfc.NfcAdapter;
 
-
-public class QtNfcBroadcastReceiver extends BroadcastReceiver
+class QtNfcBroadcastReceiver extends BroadcastReceiver
 {
-    private Context qtContext;
+    final private long qtObject;
+    final private Context qtContext;
 
-    public QtNfcBroadcastReceiver(Context context)
+    QtNfcBroadcastReceiver(long obj, Context context)
     {
+        qtObject = obj;
         qtContext = context;
         IntentFilter filter = new IntentFilter(NfcAdapter.ACTION_ADAPTER_STATE_CHANGED);
         qtContext.registerReceiver(this, filter);
     }
 
-    public void unregisterReceiver()
+    void unregisterReceiver()
     {
         qtContext.unregisterReceiver(this);
     }
 
+    @Override
     public void onReceive(Context context, Intent intent)
     {
         final int state = intent.getIntExtra(NfcAdapter.EXTRA_ADAPTER_STATE, NfcAdapter.STATE_OFF);
-        jniOnReceive(state);
+        jniOnReceive(qtObject, state);
     }
 
-    public native void jniOnReceive(int state);
+    native void jniOnReceive(long qtObject, int state);
 }
