@@ -25,19 +25,14 @@
 
 QT_BEGIN_NAMESPACE
 
-Q_DECLARE_LOGGING_CATEGORY(QT_BT_BLUEZ)
+using namespace Qt::StringLiterals;
+using namespace QtBluetoothPrivate; // for D-Bus wrappers
 
-static const QLatin1String profilePathTemplate("/qt/btsocket/%1%2/%3");
+Q_DECLARE_LOGGING_CATEGORY(QT_BT_BLUEZ)
 
 QBluetoothSocketPrivateBluezDBus::QBluetoothSocketPrivateBluezDBus()
 {
     secFlags = QBluetooth::Security::NoSecurity;
-
-    profileManager = new OrgBluezProfileManager1Interface(
-                            QStringLiteral("org.bluez"),
-                            QStringLiteral("/org/bluez"),
-                            QDBusConnection::systemBus(),
-                            this);
 }
 
 QBluetoothSocketPrivateBluezDBus::~QBluetoothSocketPrivateBluezDBus()
@@ -84,8 +79,6 @@ static QString findRemoteDevicePath(const QBluetoothAddress &address)
     if (reply.isError())
         return QString();
 
-    QString remoteDevicePath;
-
     ManagedObjectList objectList = reply.value();
     for (ManagedObjectList::const_iterator it = objectList.constBegin();
                                            it != objectList.constEnd(); ++it) {
@@ -123,7 +116,16 @@ void QBluetoothSocketPrivateBluezDBus::connectToServiceHelper(
 
     int i = 0;
     bool success = false;
-    profileUuid = uuid.toString(QUuid::WithoutBraces);
+    profileUuid = QUuid::createUuid().toString(QUuid::WithoutBraces);
+    const QString remoteDeviceUuid = uuid.toString(QUuid::WithoutBraces);
+
+    if (!profileManager) {
+        profileManager = new OrgBluezProfileManager1Interface(
+                                QStringLiteral("org.bluez"),
+                                QStringLiteral("/org/bluez"),
+                                QDBusConnection::systemBus(),
+                                this);
+    }
 
     if (profileContext) {
         qCDebug(QT_BT_BLUEZ) << "Profile context still active. close socket first.";
@@ -140,7 +142,7 @@ void QBluetoothSocketPrivateBluezDBus::connectToServiceHelper(
         // profile registration might fail in case other service uses same path
         // try 10 times and otherwise abort
 
-        profilePath = QString(profilePathTemplate).
+        profilePath = u"/qt/btsocket/%1%2/%3"_s.
                                   arg(sanitizeNameForDBus(QCoreApplication::applicationName())).
                                   arg(QCoreApplication::applicationPid()).
                                   arg(QRandomGenerator::global()->generate());
@@ -164,7 +166,7 @@ void QBluetoothSocketPrivateBluezDBus::connectToServiceHelper(
 
     QVariantMap profileOptions;
     profileOptions.insert(QStringLiteral("Role"), QStringLiteral("client"));
-    profileOptions.insert(QStringLiteral("Service"), profileUuid);
+    profileOptions.insert(QStringLiteral("Service"), remoteDeviceUuid);
     profileOptions.insert(QStringLiteral("Name"),
                           QStringLiteral("QBluetoothSocket-%1").arg(QCoreApplication::applicationPid()));
 
@@ -201,7 +203,7 @@ void QBluetoothSocketPrivateBluezDBus::connectToServiceHelper(
 
     OrgBluezDevice1Interface device(QStringLiteral("org.bluez"), remoteDevicePath,
                                     QDBusConnection::systemBus());
-    reply = device.ConnectProfile(profileUuid);
+    reply = device.ConnectProfile(remoteDeviceUuid);
     QDBusPendingCallWatcher *watcher = new QDBusPendingCallWatcher(reply, this);
     connect(watcher, &QDBusPendingCallWatcher::finished,
             this, &QBluetoothSocketPrivateBluezDBus::connectToServiceReplyHandler);
@@ -225,6 +227,23 @@ void QBluetoothSocketPrivateBluezDBus::connectToServiceReplyHandler(
         q->setSocketError(QBluetoothSocket::SocketError::HostNotFoundError);
     }
     watcher->deleteLater();
+
+    // QTBUG-82413, unregisterProfile at profileUuid,
+    // so it can be registered for new devices connecting to the same profile UUID.
+    if (profileManager) {
+        qCDebug(QT_BT_BLUEZ) << "Unregistering client profile on" << profilePath
+                             << "in connectToServiceReplyHandler() callback.";
+
+        QDBusPendingReply<> reply = profileManager->UnregisterProfile(QDBusObjectPath(profilePath));
+        reply.waitForFinished();
+        if (reply.isError())
+            qCWarning(QT_BT_BLUEZ) << "Unregister profile:" << reply.error().message();
+
+        QDBusConnection::systemBus().unregisterObject(profilePath);
+
+        delete profileManager;
+        profileManager = nullptr;
+    }
 }
 
 void QBluetoothSocketPrivateBluezDBus::connectToService(
@@ -574,18 +593,26 @@ void QBluetoothSocketPrivateBluezDBus::clearSocket()
         }
     }
 
-    QDBusPendingReply<> reply = profileManager->UnregisterProfile(QDBusObjectPath(profilePath));
-    reply.waitForFinished();
-    if (reply.isError())
-        qCWarning(QT_BT_BLUEZ) << "Unregister profile:" << reply.error().message();
-
-    QDBusConnection::systemBus().unregisterObject(profilePath);
-
     if (profileContext) {
         delete profileContext;
         profileContext = nullptr;
     }
 
+    if (profileManager) {
+        qCDebug(QT_BT_BLUEZ) << "Unregistering client profile on" << profilePath
+                             << "in clearSocket().";
+
+        QDBusPendingReply<> reply = profileManager->UnregisterProfile(QDBusObjectPath(profilePath));
+        reply.waitForFinished();
+        if (reply.isError())
+            qCWarning(QT_BT_BLUEZ) << "Unregister profile:" << reply.error().message();
+
+        QDBusConnection::systemBus().unregisterObject(profilePath);
+
+        delete profileManager;
+        profileManager = nullptr;
+    }
+
     remoteDevicePath.clear();
     profileUuid.clear();
     profilePath.clear();
